CONFIG_FILE = '.manageacloud'

MAC_FILE = '.manageacloud'
AUTH_SECTION = 'auth'
USER_OPTION = 'user'
APIKEY_OPTION = 'apikey'
AUTH_ERROR = 'auth_error'
NO_ERROR = 'no_error'

RELEASE_ANY = "any"
CONFIGURATION_FAILED = "Configuration Error"
CREATION_FAILED = "Creation failed"

AUTH_ERROR_EXIT_CODE = 2
EXCEPTION_EXIT_CODE = 3

MACFILE_ON_FAILURE_DESTROY_ALL = "destroy_all"
MACFILE_ON_FAILURE_DESTROY_OTHERS = "destroy_others"